body {
  display: flex;
  height: 100vh;
  margin: 0px;
  overflow-x: hidden;
  overflow-y: hidden;
}

.bk-root .bk, .bk-root .bk:before, .bk-root .bk:after {
  font-family: inherit;
}

img {
  max-width: 100%;
}

#container {
  padding:0px;
  height:100vh;
  width: 100vw;
  max-width: 100vw;
}

#sidebar .mdc-list {
  padding-left: 5px;
  padding-right: 5px;
}

.mdc-drawer-app-content {
  flex: auto;
  position: relative;
  overflow: hidden;
}

.mdc-drawer {
  background: #FAFAFA; /* GRAY 50 */
}

.mdc-drawer-app-content {
  margin-left: 0 !important;
}

.title-bar {
  display: contents;
  justify-content: center;
  align-content: center;
  width: 100%;
}

.mdc-top-app-bar .bk-menu {
  color: black
}

.app-header {
  display: contents;
  padding-left: 10px;
  font-size: 1.25em;
}

img.app-logo {
  padding-right: 10px;
  font-size: 28px;
  height: 30px;
  max-width: inherit;
  padding-top: 12px;
  padding-bottom: 6px;
}

#app-title {
  padding-right: 12px;
  padding-left: 12px;
}

.title {
  color: #fff;
  text-decoration: none;
  text-decoration-line: none;
  text-decoration-style: initial;
  text-decoration-color: initial;
  font-weight: 400;
  font-size: 1.25em;
  line-height: 2em;
  white-space: nowrap;
}

.main-content {
  overflow-y: scroll;
  overflow-x: auto;
}

#header {
  position: absolute;
  z-index: 7;
}

#header-items {
  width: 100%;
  margin-left:15px;
}

.pn-busy-container {
  align-items: center;
  justify-content: center;
  display: flex;
}

.mdc-drawer__content {
  overflow-x: hidden;
}
.mdc-drawer__content, .main-content {
  padding: 12px;
}

.main-content {
  height: calc(100vh - 88px);
  max-height: calc(100vh - 88px);
  padding-right: 32px;
}

button.mdc-button.mdc-card-button {
  color: transparent;
  height: 50px;
}

p.bk.mdc-button {
  display: none;
}

div.bk.mdc-card {
  border-radius: 0px
}

.mdc-card .bk.card-header {
  display: flex;
}

.bk.mdc-card-title {
  font-size: 1.4em;
  font-weight: bold;
  align-items: center;
  display: flex !important;
  position: relative !important;
}

.bk.mdc-card-title:nth-child(2) {
  margin-left: -1.4em;
}

.pn-modal {
  overflow-y: scroll;
  width: 100%;
  display: none;
  position: absolute;
  top: 0;
  left: 0;
}

.pn-modal-content {
  background-color: #fefefe;
  margin: auto;
  margin-top: 25px;
  margin-bottom: 25px;
  padding: 15px 20px 20px 20px;
  border: 1px solid #888;
  width: 80% !important;
}

.pn-modal-close {
  position: absolute;
  right: 25px;
  z-index: 100;
}

.pn-modal-close:hover,
.pn-modal-close:focus {
  color: #000;
  text-decoration: none;
  cursor: pointer;
}
